@@ -55,6 +55,19 @@ PRESET_SLOGANS = [
     "Making \"git pull\" fun again, since 2010.",
     "like a compact disc to the head!",
     "along with his mechanical ass-kicking leg.",
+    "TIME FOR GROUP HUG.",
+    "GLORIOUS VICTORY",
+    "the sock ruse was a... DISTACTION",
+    "wigeons have my car keys.",
+    "life is like a zombie in my head.",
+    "head to the nearest ENTRANCE and immediately call YOUR MUTANT FIRE DANCING MOON POSSE",
+    "it's got what dents crave.",
+    "it has lightsabers.",
+    "enemy of #scannability.",
+    "T-rex rules the school.",
+    "GET BLUE SPHERES",
+    "not affiliated with @sandersch's nipple.",
+    "we're sitti. D next to toy fish",
     ]
 
 def main():
@@ -63,8 +76,8 @@ def main():
     """
     parser = OptionParser()
     parser.add_option("-c", "--config",
-        help="specify an alternative config file to use", action="store",
-        type="string", dest="config_filename", metavar="FILE")
+        help="specify an alternative config dir to use", action="store",
+        type="string", dest="config_dirname", metavar="FILE")
     parser.add_option("-s", "--slogans",
         help="specify an alternative slogans file to use", action="store",
         type="string", dest="slogans_filename", metavar="FILE")
@@ -85,8 +98,8 @@ def main():
         user_slogans_file = os.path.join(os.path.expanduser("~"),
                                              ".identicurse_slogans")
     
-    if options.config_filename is not None:
-        additional_config['config_filename'] = options.config_filename
+    if options.config_dirname is not None:
+        additional_config['config_dirname'] = options.config_dirname
 
     try:
         user_slogans_raw = open(user_slogans_file).read()
@@ -95,7 +108,7 @@ def main():
         slogans = user_slogans
     except IOError:
         slogans = PRESET_SLOGANS
-    print "Welcome to IdentiCurse 0.7.3 (Glastonbury) - %s" % (random.choice(slogans))
+    print "Welcome to IdentiCurse 0.8.2 (Holyhead) - %s" % (random.choice(slogans))
     IdentiCurse(additional_config)
 
 def colour_check():
