@@ -21,16 +21,18 @@ offset_regex = re.compile("[+-][0-9]{4}")
 base_url_regex = re.compile("(http(s|)://.+?)/.*")
 title_regex = re.compile("\<title\>(.*)\<\/title\>")
 ur1_regex = re.compile("Your ur1 is: <a.+?>(http://ur1\.ca/[0-9A-Za-z]+)")
-url_regex = re.compile("http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+")
+url_regex = re.compile("(?:ht|f)tp[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+")
+domain_regex = re.compile("http(s|)://(www\.|)(.+?)(/.*|)$")
 
-def notice_datetime(notice):
+
+def normalise_datetime(timestring):
     locale.setlocale(locale.LC_TIME, 'C')  # hacky fix because statusnet uses english timestrings regardless of locale
-    created_at_no_offset = offset_regex.sub("+0000", notice['created_at'])
+    datetime_no_offset = offset_regex.sub("+0000", timestring)
     attempts = 10
     while attempts > 0:
         attempts -= 1
         try:
-            normalised_datetime = datetime.datetime.strptime(created_at_no_offset, DATETIME_FORMAT) + utc_offset(notice['created_at'])
+            normalised_datetime = datetime.datetime.strptime(datetime_no_offset, DATETIME_FORMAT) + utc_offset(timestring)
             break
         except ValueError:  # something else changed the locale, and Python threw a hissy fit
             pass
@@ -215,15 +217,22 @@ def set_terminal_title(title_text):
 
 def split_entities(raw_notice_text):
     entities = [{"text":"", "type":"plaintext"}]
-    raw_notice_text = " " + raw_notice_text
+    raw_notice_text = " " + raw_notice_text + " "
     char_index = 0
     while char_index < len(raw_notice_text):
         if entities[-1]['type'] != "plaintext" and not raw_notice_text[char_index].isalnum() and not raw_notice_text[char_index] in [".", "_", "-"]:
-            entities.append ({"text":"", "type":"plaintext"})
+            next_entity_text = ""
+            for i in xrange(len(entities[-1]['text'])):
+                if len(entities[-1]['text']) > 1 and entities[-1]['text'][-1] in [".", "-"]:
+                    next_entity_text += entities[-1]['text'][-1]
+                    entities[-1]['text'] = entities[-1]['text'][:-1]
+                else:
+                    break
+            entities.append ({"text":next_entity_text, "type":"plaintext"})
         if (raw_notice_text[char_index] in string.whitespace or raw_notice_text[char_index] in string.punctuation) and char_index < (len(raw_notice_text) - 2):
             entities[-1]['text'] += raw_notice_text[char_index]
             char_index += 1
-            if raw_notice_text[char_index] in ["@", "!", "#"] and (raw_notice_text[char_index+1].isalnum() or (raw_notice_text[char_index+1] in [".", "_", "-"])):
+            if raw_notice_text[char_index] in ["@", "!", "#"] and (raw_notice_text[char_index+1].isalnum() or (raw_notice_text[char_index+1] in [".", "_", "-"])) and (not raw_notice_text[char_index-1] in ["@", "!", "#", ".", "_", "-"]) and (not raw_notice_text[char_index-1].isalnum()):
                 if raw_notice_text[char_index] == "@":
                     entities.append({"text":"@", "type":"user"})
                 elif raw_notice_text[char_index] == "!":
@@ -234,8 +243,14 @@ def split_entities(raw_notice_text):
         else:
             entities[-1]['text'] += raw_notice_text[char_index]
             char_index += 1
+    # strip the extra space that was prepended
     if entities[0]['text'] == " ":
         entities = entities[1:]
     else:
         entities[0]['text'] = entities[0]['text'][1:]
+    # and the one that was appended
+    if entities[-1]['text'] == " ":
+        entities = entities[:-1]
+    else:
+        entities[-1]['text'] = entities[-1]['text'][:-1]
     return entities
