@@ -16,7 +16,8 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 from helpers import DATETIME_FORMAT
-import os.path, re, sys, threading, datetime, locale, helpers, curses, random, identicurse, config, httplib
+import os.path, re, sys, threading, datetime, locale, curses, random, httplib
+import identicurse, config, helpers
 from operator import itemgetter
 from statusnet import StatusNetError
 
@@ -110,6 +111,7 @@ class Tab(object):
         self.page = 1
         self.search_highlight_line = -1
         self.active = False
+        self.paused = False
         
     def prevpage(self, n=1):
         if hasattr(self, "timeline"):
@@ -207,6 +209,8 @@ class Tab(object):
             remaining_line_length = maxx - 2
             try:
                 for (part, attr) in line:
+                    if attr == identicurse.colour_fields["pause_line"]:  # we want pause lines to fill the width
+                        self.window.addstr("-"*(remaining_line_length-1), curses.color_pair(identicurse.colour_fields['pause_line']))
                     if line_num == self.search_highlight_line:
                         remaining_line_length -= len(part)
                         self.window.addstr(part, curses.color_pair(identicurse.colour_fields['search_highlight']))
@@ -266,7 +270,22 @@ class Timeline(Tab):
         
         Tab.__init__(self, window)
 
+    def update_name(self):
+        if self.page > 1:
+            self.name = self.basename + "+%d" % (self.page - 1)
+        else:
+            self.name = self.basename
+
+        if self.paused:
+            self.name = self.name + " (paused)"
+
     def update(self):
+        self.update_name()
+
+        if self.paused:
+            self.update_buffer()
+            return
+
         get_count = config.config['notice_limit']
 
         if self.prev_page != self.page:
@@ -301,15 +320,18 @@ class Timeline(Tab):
             raw_timeline = self.conn.search(self.type_params['query'], page=self.page, standardise=True, since_id=last_id)
         elif self.timeline_type == "context":
             raw_timeline = []
-            if last_id == 0:  # don't run this if we've already filled the timeline
-                next_id = self.type_params['notice_id']
-                while next_id is not None:
-                    notice = self.conn.statuses_show(id=next_id)
-                    raw_timeline.append(notice)
-                    if "retweeted_status" in notice:
-                        next_id = notice['retweeted_status']['id']
-                    else:
-                        next_id = notice['in_reply_to_status_id']
+            if "conversation_id" in self.type_params:  # try to do it the new way
+                raw_timeline = self.conn.statusnet_conversation(self.type_params['conversation_id'], count=get_count, since_id=last_id, page=self.page)
+            else:
+                if last_id == 0:  # don't run this if we've already filled the timeline
+                    next_id = self.type_params['notice_id']
+                    while next_id is not None:
+                        notice = self.conn.statuses_show(id=next_id)
+                        raw_timeline.append(notice)
+                        if "retweeted_status" in notice:
+                            next_id = notice['retweeted_status']['id']
+                        else:
+                            next_id = notice['in_reply_to_status_id']
 
         self.prev_page = self.page
 
@@ -317,16 +339,28 @@ class Timeline(Tab):
         old_ids = [n['id'] for n in self.timeline]
 
         for notice in raw_timeline:
-            notice["ic__raw_datetime"] = helpers.notice_datetime(notice)
+            notice["ic__raw_datetime"] = helpers.normalise_datetime(notice['created_at'])
             notice["ic__from_web"] = True
             passes_filters = True
             if notice['id'] in old_ids:
                 passes_filters = False
                 continue
-            for filter_item in config.config['filters']:
-                if filter_item.lower() in notice['text'].lower():
-                    passes_filters = False
-                    break
+            if hasattr(config.session_store, "muted_conversations") and notice['statusnet_conversation_id'] in config.session_store.muted_conversations:
+                passes_filters = False
+                continue
+            if config.config["hide_activities"] and ("source" in notice) and (notice["source"] == "activity"):
+                passes_filters = False
+                continue
+            if config.config["filter_mode"] == "regex":
+                for filter_item in config.config['filters']:
+                    if filter_item.search(notice['text']) is not None:
+                        passes_filters = False
+                        break
+            else:
+                for filter_item in config.config['filters']:
+                    if filter_item.lower() in notice['text'].lower():
+                        passes_filters = False
+                        break
             if passes_filters:
                 if (not self.timeline_type in ["direct", "sentdirect"]) and notice["source"] == "ostatus" and config.config['expand_remote'] and "attachments" in notice:
                     import urllib2
@@ -347,22 +381,20 @@ class Timeline(Tab):
 
         if self.timeline_type in ["direct", "mentions"]:  # alert on changes to these. maybe config option later?
             if (len(self.timeline) > 0) and (len(temp_timeline) > 0):  # only fire when there's new stuff _and_ we've already got something in the timeline
-                curses.flash()
+                if config.config['notify'] == 'flash':
+                    curses.flash()
+                elif config.config['notify'] == 'beep':
+                    curses.beep()
 
         if len(self.timeline) == 0:
             self.timeline = temp_timeline[:]
         else:
             self.timeline = temp_timeline + self.timeline
-            if len(self.timeline) > get_count:  # truncate long timelines
+            if self.timeline_type != "context" and len(self.timeline) > get_count:  # truncate long timelines
                 self.timeline = self.timeline[:get_count]
 
         self.timeline.sort(key=itemgetter('id'), reverse=True)
 
-        if self.page > 1:
-            self.name = self.basename + "+%d" % (self.page - 1)
-        else:
-            self.name = self.basename
-
         self.search_highlight_line = -1
 
         self.update_buffer()
@@ -379,7 +411,17 @@ class Timeline(Tab):
                 user_string = "%s -> %s" % (n["sender"]["screen_name"], n["recipient"]["screen_name"])
                 source_msg = ""
             else:
-                user_string = "%s" % (n["user"]["screen_name"])
+                atless_reply = False
+                if "in_reply_to_screen_name" in n and n["in_reply_to_screen_name"] is not None:
+                    atless_reply = True
+                    for entity in helpers.split_entities(n["text"]):
+                        if entity["type"] == "user" and entity["text"][1:].lower() == n["in_reply_to_screen_name"].lower():
+                            atless_reply = False
+                            break
+                if atless_reply:
+                    user_string = "%s -> %s" % (n["user"]["screen_name"], n["in_reply_to_screen_name"])
+                else:
+                    user_string = "%s" % (n["user"]["screen_name"])
                 raw_source_msg = "from %s" % (n["source"])
                 source_msg = self.html_regex.sub("", raw_source_msg)
             if "in_reply_to_status_id" in n and n["in_reply_to_status_id"] is not None:
@@ -394,7 +436,7 @@ class Timeline(Tab):
                         user_string += " +"
                     else:
                         source_msg += " [+]"
-            datetime_notice = helpers.notice_datetime(n)
+            datetime_notice = helpers.normalise_datetime(n["created_at"])
             time_msg = helpers.format_time(helpers.time_since(datetime_notice), short_form=True)
             metadata_string = time_msg + " " + user_string
             if config.config["show_source"]:
@@ -415,12 +457,21 @@ class Timeline(Tab):
                     repeating_user = n["user"]["screen_name"]
                     n = n["retweeted_status"]
                 from_user = n["user"]["screen_name"]
+                atless_reply = False
+                if "in_reply_to_screen_name" in n and n["in_reply_to_screen_name"] is not None:
+                    atless_reply = True
+                    for entity in helpers.split_entities(n["text"]):
+                        if entity["type"] == "user" and entity["text"][1:].lower() == n["in_reply_to_screen_name"].lower():
+                            atless_reply = False
+                            break
+                if atless_reply:
+                    to_user = n["in_reply_to_screen_name"]
                 raw_source_msg = "from %s" % (n["source"])
                 source_msg = self.html_regex.sub("", raw_source_msg)
                 repeat_msg = ""
                 if n["in_reply_to_status_id"] is not None:
                     source_msg += " [+]"
-            datetime_notice = helpers.notice_datetime(n)
+            datetime_notice = helpers.normalise_datetime(n["created_at"])
 
             time_msg = helpers.format_time(helpers.time_since(datetime_notice), short_form=True)
 
@@ -428,6 +479,10 @@ class Timeline(Tab):
                 if not user in config.session_store.user_cache:
                     config.session_store.user_cache[user] = random.choice(identicurse.base_colours.items())[1]
            
+            if "ic__paused_on" in n and c != 1:
+                self.buffer.append([("-", identicurse.colour_fields["pause_line"])])
+                self.buffer.append([("", identicurse.colour_fields["none"])])
+
             # Build the line
             line = []
 
@@ -513,7 +568,8 @@ class Timeline(Tab):
                         if entity['type'] in ['user', 'group', 'tag']:
                             entity_text_no_symbol = entity['text'][1:]
                             cache = getattr(config.session_store, '%s_cache' % (entity['type']))
-                            cache[entity_text_no_symbol] = random.choice(identicurse.base_colours.items())[1]
+                            if not entity_text_no_symbol in cache:
+                                cache[entity_text_no_symbol] = random.choice(identicurse.base_colours.items())[1]
                             if config.config['%s_rainbow' % (entity['type'])]:
                                 line.append((entity['text'], cache[entity_text_no_symbol]))
                             else:
@@ -584,9 +640,7 @@ class Profile(Tab):
                 self.profile['following'] = "No"
 
             # create this field specially
-            locale.setlocale(locale.LC_TIME, 'C')  # hacky fix because statusnet uses english timestrings regardless of locale
-            datetime_joined = datetime.datetime.strptime(self.profile['created_at'], DATETIME_FORMAT)
-            locale.setlocale(locale.LC_TIME, '') # other half of the hacky fix
+            datetime_joined = helpers.normalise_datetime(self.profile['created_at'])
             days_since_join = helpers.single_unit(helpers.time_since(datetime_joined), "days")['days']
             self.profile['notices_per_day'] = "%0.2f" % (float(self.profile['statuses_count']) / days_since_join)
 
