@@ -44,14 +44,14 @@ class Textbox(textpad.Textbox):
             insert = False
             ch = self.win.getch()
 
-            if ch == 127:
-                self.do_command(263)
+            if ch == curses.ascii.DEL:
+                self.do_command(curses.ascii.BS)
             elif ch == curses.KEY_ENTER or ch == 10:
                 break
-            elif ch == 27:
+            elif ch == curses.ascii.ESC:
                 abort = True
                 break
-            elif ch == 9:
+            elif ch == curses.ascii.TAB:
                 cursor_position = self.win.getyx()
                 x = cursor_position[1]
                 last_word = ""
@@ -101,7 +101,7 @@ class Textbox(textpad.Textbox):
                     else:
                         possible_guesses = helpers.find_fuzzy_matches(last_word, guess_source)
                         common_guess = helpers.find_longest_common_start(possible_guesses)
-                        if len(common_guess) != 0:
+                        if len(common_guess) != 0 and len(helpers.find_fuzzy_matches(last_word, [common_guess,])) > 0:
                             self.win.move(self.win.getyx()[0], self.win.getyx()[1]-len(last_word))
                             for i in xrange(len(last_word)):
                                 self.delch()
@@ -131,6 +131,38 @@ class Textbox(textpad.Textbox):
                 self.delch()
             elif ch == curses.KEY_DC or ch == curses.ascii.ctrl(ord("d")):
                 self.delch()
+            elif ch == curses.ascii.ctrl(ord("u")):  # delete entire line up to the cursor
+                cursor_y, cursor_x = self.win.getyx()
+                self.win.move(cursor_y, 0)
+                for char_count in xrange(0, cursor_x):
+                    self.delch()
+            elif ch == curses.ascii.ctrl(ord("w")):  # delete all characters before the current one until the beginning of the word
+                cursor_y, cursor_x = self.win.getyx()
+                x, y = cursor_x, cursor_y
+                only_spaces_so_far = True
+                while True:
+                    if x == 0:
+                        if y == 0:
+                            break
+                        else:
+                            y -= 1
+                            x = self.maxx
+                    else:
+                        x -= 1
+                    if curses.ascii.ascii(self.win.inch(y, x)) != ord(" "):
+                        self.win.move(y, x)
+                        self.delch()
+                        only_spaces_so_far = False
+                    else:
+                        if only_spaces_so_far:
+                            self.win.move(y, x)
+                            self.delch()
+                        else:
+                            if x == self.maxx:
+                                self.win.move(y + 1, 0)
+                            else:
+                                self.win.move(y, x + 1)
+                            break
             elif not ch:
                 continue
             elif not self.do_command(ch):
